@@ -14,10 +14,8 @@
 #include <execinfo.h>
 #include <csignal>
 #include <sys/poll.h>
-#include <sys/prctl.h>
 #include <sys/stat.h>
 #include <sys/uio.h>
-#include <sys/vfs.h>
 #include <unistd.h>
 
 #include <atomic>
@@ -38,11 +36,10 @@
 #include "Common.hpp"
 #include "Log.hpp"
 
-/// Flag to request hard termination.
 std::atomic<bool> TerminationFlag(false);
-/// Flag to request dumping of all internal state
 std::atomic<bool> DumpGlobalState(false);
-/// Flag to request WSD to notify clients and shutdown.
+
+#ifndef MOBILEAPP
 std::atomic<bool> ShutdownRequestFlag(false);
 
 std::mutex SigHandlerTrap;
@@ -313,4 +310,6 @@ namespace SigUtil
     }
 }
 
+#endif // !MOBILEAPP
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
