@@ -1,3 +1,4 @@
+/* -*- js-indent-level: 8 -*- */
 /*
 * Control.Menubar
 */
@@ -17,6 +18,7 @@ L.Control.Menubar = L.Control.extend({
 			{name: _UNO('.uno:PickList', 'text'), id: 'file', type: 'menu', menu: [
 				{name: _UNO('.uno:Save', 'text'), id: 'save', type: 'action'},
 				{name: _UNO('.uno:SaveAs', 'text'), id: 'saveas', type: 'action'},
+				{name: _('Share...'), id:'shareas', type: 'action'},
 				{name: _UNO('.uno:Print', 'text'), id: 'print', type: 'action'},
 				{name: _('See revision history'), id: 'rev-history', type: 'action'},
 				{name: _('Download as'), id: 'downloadas', type: 'menu', menu: [
@@ -25,6 +27,7 @@ L.Control.Menubar = L.Control.extend({
 					{name: _('Word 2003 Document (.doc)'), id: 'downloadas-doc', type: 'action'},
 					{name: _('Word Document (.docx)'), id: 'downloadas-docx', type: 'action'},
 					{name: _('Rich Text (.rtf)'), id: 'downloadas-rtf', type: 'action'}]},
+				{name: _('Sign document'), id: 'signdocument', type: 'action'},
 				{type: 'separator'},
 				{uno: '.uno:SetDocumentProperties'}
 			]},
@@ -63,7 +66,8 @@ L.Control.Menubar = L.Control.extend({
 			]
 			},
 			{name: _UNO('.uno:InsertMenu', 'text'), type: 'menu', menu: [
-				{name: _UNO('.uno:InsertGraphic', 'text'), id: 'insertgraphic', type: 'action'},
+				{name: _('Local Image...'), id: 'insertgraphic', type: 'action'},
+				{name: _UNO('.uno:InsertGraphic', 'text'), id: 'insertgraphicremote', type: 'action'},
 				{name: _UNO('.uno:InsertAnnotation', 'text'), id: 'insertcomment', type: 'action'},
 				{uno: '.uno:InsertObjectChart'},
 				{type: 'separator'},
@@ -257,7 +261,8 @@ L.Control.Menubar = L.Control.extend({
 				{name: _('Reset zoom'), id: 'zoomreset', type: 'action'}]
 			},
 			{name: _UNO('.uno:InsertMenu', 'presentation'), type: 'menu', menu: [
-				{name: _UNO('.uno:InsertGraphic', 'presentation'), id: 'insertgraphic', type: 'action'},
+				{name: _('Local Image...'), id: 'insertgraphic', type: 'action'},
+				{name: _UNO('.uno:InsertGraphic', 'presentation'), id: 'insertgraphicremote', type: 'action'},
 				{name: _UNO('.uno:InsertAnnotation', 'presentation'), id: 'insertcomment', type: 'action'},
 				{uno: '.uno:InsertObjectChart'},
 				{type: 'separator'},
@@ -333,10 +338,12 @@ L.Control.Menubar = L.Control.extend({
 				{name: _UNO('.uno:FullScreen', 'spreadsheet'), id: 'fullscreen', type: 'action'}
 			]},
 			{name: _UNO('.uno:InsertMenu', 'spreadsheet'), type: 'menu', menu: [
-				{name: _UNO('.uno:InsertGraphic', 'spreadsheet'), id: 'insertgraphic', type: 'action'},
+				{name: _('Local Image...'), id: 'insertgraphic', type: 'action'},
+				{name: _UNO('.uno:InsertGraphic', 'spreadsheet'), id: 'insertgraphicremote', type: 'action'},
 				{uno: '.uno:InsertObjectChart'},
 				{name: _UNO('.uno:InsertAnnotation', 'spreadsheet'), id: 'insertcomment', type: 'action'},
 				{type: 'separator'},
+				{uno: '.uno:HyperlinkDialog'},
 				{uno: '.uno:InsertSymbol'}
 			]},
 			{name: _UNO('.uno:FormatMenu', 'spreadsheet'), type: 'menu', menu: [
@@ -544,10 +551,13 @@ L.Control.Menubar = L.Control.extend({
 				// animate mobile menu
 				$mainMenuState.change(function() {
 					var $menu = $('#main-menu');
+					var $nav = $menu.parent();
 					if (this.checked) {
+						$nav.css({height: 'initial', bottom: 33});
 						$menu.hide().slideDown(250, function() { $menu.css('display', ''); });
 					} else {
 						$menu.show().slideUp(250, function() { $menu.css('display', ''); });
+						$nav.css({height:0, bottom: ''});
 					}
 				});
 				// hide mobile menu beforeunload
@@ -671,6 +681,8 @@ L.Control.Menubar = L.Control.extend({
 			this._map.save(true, true);
 		} else if (id === 'saveas') {
 			this._map.fire('postMessage', {msgId: 'UI_SaveAs'});
+		} else if (id === 'shareas') {
+			this._map.fire('postMessage', {msgId: 'UI_Share'});
 		} else if (id === 'print') {
 			this._map.print();
 		} else if (id.startsWith('downloadas-')) {
@@ -679,10 +691,14 @@ L.Control.Menubar = L.Control.extend({
 			fileName = fileName.substr(0, fileName.lastIndexOf('.'));
 			fileName = fileName === '' ? 'document' : fileName;
 			this._map.downloadAs(fileName + '.' + format, format);
+		} else if (id === 'signdocument') {
+			this._map.showSignDocument();
 		} else if (id === 'insertcomment') {
 			this._map.insertComment();
 		} else if (id === 'insertgraphic') {
 			L.DomUtil.get('insertgraphic').click();
+		} else if (id === 'insertgraphicremote') {
+			this._map.fire('postMessage', {msgId: 'UI_InsertGraphic'});
 		} else if (id === 'zoomin' && this._map.getZoom() < this._map.getMaxZoom()) {
 			this._map.zoomIn(1);
 		} else if (id === 'zoomout' && this._map.getZoom() > this._map.getMinZoom()) {
@@ -690,28 +706,7 @@ L.Control.Menubar = L.Control.extend({
 		} else if (id === 'zoomreset') {
 			this._map.setZoom(this._map.options.zoom);
 		} else if (id === 'fullscreen') {
-			if (!document.fullscreenElement &&
-				!document.mozFullscreenElement &&
-				!document.msFullscreenElement &&
-				!document.webkitFullscreenElement) {
-				if (document.documentElement.requestFullscreen) {
-					document.documentElement.requestFullscreen();
-				} else if (document.documentElement.msRequestFullscreen) {
-					document.documentElement.msRequestFullscreen();
-				} else if (document.documentElement.mozRequestFullScreen) {
-					document.documentElement.mozRequestFullScreen();
-				} else if (document.documentElement.webkitRequestFullscreen) {
-					document.documentElement.webkitRequestFullscreen(Element.ALLOW_KEYBOARD_INPUT);
-				}
-			} else if (document.exitFullscreen) {
-				document.exitFullscreen();
-			} else if (document.msExitFullscreen) {
-				document.msExitFullscreen();
-			} else if (document.mozCancelFullScreen) {
-				document.mozCancelFullScreen();
-			} else if (document.webkitExitFullscreen) {
-				document.webkitExitFullscreen();
-			}
+			L.toggleFullScreen();
 		} else if (id === 'fullscreen-presentation' && this._map.getDocType() === 'presentation') {
 			this._map.fire('fullscreen');
 		} else if (id === 'insertpage') {
@@ -733,8 +728,12 @@ L.Control.Menubar = L.Control.extend({
 			this._map.fire('postMessage', {msgId: 'rev-history', args: {Deprecated: true}});
 			this._map.fire('postMessage', {msgId: 'UI_FileVersions'});
 		} else if (id === 'closedocument') {
-			this._map.fire('postMessage', {msgId: 'close', args: {EverModified: this._map._everModified, Deprecated: true}});
-			this._map.fire('postMessage', {msgId: 'UI_Close', args: {EverModified: this._map._everModified}});
+			if (window.ThisIsAMobileApp) {
+				window.webkit.messageHandlers.lool.postMessage('BYE', '*');
+			} else {
+				this._map.fire('postMessage', {msgId: 'close', args: {EverModified: this._map._everModified, Deprecated: true}});
+				this._map.fire('postMessage', {msgId: 'UI_Close', args: {EverModified: this._map._everModified}});
+			}
 			this._map.remove();
 		} else if (id === 'repair') {
 			this._map._socket.sendMessage('commandvalues command=.uno:DocumentRepair');
@@ -768,7 +767,7 @@ L.Control.Menubar = L.Control.extend({
 			self._executeAction(item);
 		}
 
-		if ($(item).data('id') !== 'insertcomment')
+		if (!L.Browser.mobile && $(item).data('id') !== 'insertcomment')
 			self._map.focus();
 	},
 
@@ -779,6 +778,9 @@ L.Control.Menubar = L.Control.extend({
 			if (menu[i].id === 'about' && (L.DomUtil.get('about-dialog') === null)) {
 				continue;
 			}
+			if (menu[i].id === 'signdocument' && (L.DomUtil.get('document-signing-bar') === null)) {
+				continue;
+			}
 
 			if (this._map._permission === 'readonly' && menu[i].type === 'menu') {
 				var found = false;
@@ -808,6 +810,12 @@ L.Control.Menubar = L.Control.extend({
 			if (menu[i].id === 'saveas' && this._map['wopi'].UserCanNotWriteRelative)
 				continue;
 
+			if (menu[i].id === 'shareas' && !this._map['wopi'].EnableShare)
+				continue;
+
+			if (menu[i].id === 'insertgraphicremote' && !this._map['wopi'].EnableInsertRemoteImage)
+				continue;
+
 			if (menu[i].id && menu[i].id.startsWith('fullscreen-presentation') && this._map['wopi'].HideExportOption)
 				continue;
 
