@@ -1,3 +1,4 @@
+/* -*- js-indent-level: 8 -*- */
 /*
  * L.Control.Tabs is used to switch sheets in Calc
  */
@@ -29,6 +30,13 @@ L.Control.Tabs = L.Control.extend({
 		var map = this._map;
 		var docContainer = map.options.documentContainer;
 		this._tabsCont = L.DomUtil.create('div', 'spreadsheet-tabs-container', docContainer.parentElement);
+		L.DomEvent.on(this._tabsCont, 'touchstart',
+			function (e) {
+				if (e && e.touches.length > 1) {
+					L.DomEvent.preventDefault(e);
+				}
+			},
+			this);
 
 		$.contextMenu({
 			selector: '.spreadsheet-tab',
@@ -64,7 +72,7 @@ L.Control.Tabs = L.Control.extend({
 								var nPos = parseInt(options.$trigger.attr('id').split('spreadsheet-tab')[1]);
 								vex.dialog.open({
 									message: _('Enter new sheet name'),
-									input: '<input name="sheetname" type="text" required />',
+									input: '<input name="sheetname" type="text" value="' + options.$trigger.text() + '" required />',
 									callback: function(data) {
 										map.renamePage(data.sheetname, nPos);
 									}
@@ -120,7 +128,7 @@ L.Control.Tabs = L.Control.extend({
 						continue;
 					var id = 'spreadsheet-tab' + i;
 					var tab = L.DomUtil.create('div', 'spreadsheet-tab', ssTabScroll);
-					tab.textContent = e.partNames[i];
+					tab.innerHTML = e.partNames[i];
 					tab.id = id;
 
 					L.DomEvent
