@@ -1,3 +1,4 @@
+/* -*- js-indent-level: 8 -*- */
 /*
  * Document permission handler
  */
@@ -6,7 +7,9 @@ L.Map.include({
 		this._permission = perm;
 		if (perm === 'edit') {
 			this._socket.sendMessage('requestloksession');
-			this.dragging.disable();
+			if (!L.Browser.touch) {
+				this.dragging.disable();
+			}
 		}
 		else if (perm === 'view' || perm === 'readonly') {
 			this.dragging.enable();
