@@ -1,3 +1,4 @@
+/* -*- js-indent-level: 8 -*- */
 /*
  * L.ClipboardContainer is used to overlay the hidden clipbaord container on the map
  */
@@ -17,9 +18,7 @@ L.ClipboardContainer = L.Layer.extend({
 		L.DomEvent.on(this._textArea, 'copy cut paste ' +
 		              'keydown keypress keyup ' +
 		              'compositionstart compositionupdate compositionend textInput',
-		              this._map._handleDOMEvent, this._map)
-			  .on(this._textArea, 'focus', this.onGotFocus, this)
-			  .on(this._textArea, 'blur', this.onLostFocus, this);
+		              this._map._handleDOMEvent, this._map);
 	},
 
 	onRemove: function () {
@@ -30,24 +29,10 @@ L.ClipboardContainer = L.Layer.extend({
 		L.DomEvent.off(this._textArea, 'copy cut paste ' +
 		               'keydown keypress keyup ' +
 		               'compositionstart compositionupdate compositionend textInput',
-		               this._map._handleDOMEvent, this._map)
-			  .off(this._textArea, 'focus', this.onGotFocus, this)
-			  .off(this._textArea, 'blur', this.onLostFocus, this);
-	},
-
-	onGotFocus: function () {
-		this.setLatLng(this._map._docLayer._updateCursorPos());
-	},
-
-	onLostFocus: function () {
-		this._map.removeLayer(this._map._docLayer._cursorMarker);
+		               this._map._handleDOMEvent, this._map);
 	},
 
 	focus: function(focus) {
-		if (this._map._permission !== 'edit') {
-			return;
-		}
-
 		if (focus === false) {
 			this._textArea.blur();
 		} else {
@@ -90,6 +75,18 @@ L.ClipboardContainer = L.Layer.extend({
 		this._textArea.setAttribute('spellcheck', 'false');
 	},
 
+	activeElement: function () {
+		return this._textArea;
+	},
+
+	showCursor: function () {
+		this._map.addLayer(this._map._docLayer._cursorMarker);
+
+		// move the hidden input field with the cursor
+		var cursorPos = this._map._docLayer._visibleCursor.getNorthWest();
+		L.DomUtil.setPosition(this._container, this._map.latLngToLayerPoint(L.latLng(cursorPos)).round());
+	},
+
 	_setPos: function (pos) {
 		L.DomUtil.setPosition(this._container, pos);
 	}
