@@ -242,9 +242,6 @@ L.CalcTileLayer = L.TileLayer.extend({
 			command.height = parseInt(strTwips[3]);
 			command.part = this._selectedPart;
 		}
-		if (this._docType === 'text') {
-			command.part = 0;
-		}
 		var topLeftTwips = new L.Point(command.x, command.y);
 		var offset = new L.Point(command.width, command.height);
 		var bottomRightTwips = topLeftTwips.add(offset);
@@ -256,11 +253,7 @@ L.CalcTileLayer = L.TileLayer.extend({
 		var visibleBottomRight = this._latLngToTwips(this._map.getBounds().getSouthEast());
 		var visibleArea = new L.Bounds(visibleTopLeft, visibleBottomRight);
 
-		var tilePositionsX = '';
-		var tilePositionsY = '';
-		var oldWireIds = '';
 		var needsNewTiles = false;
-
 		for (var key in this._tiles) {
 			var coords = this._tiles[key].coords;
 			var tileTopLeft = this._coordsToTwips(coords);
@@ -274,23 +267,6 @@ L.CalcTileLayer = L.TileLayer.extend({
 					this._tiles[key]._invalidCount = 1;
 				}
 				if (visibleArea.intersects(bounds)) {
-					if (tilePositionsX !== '') {
-						tilePositionsX += ',';
-					}
-					tilePositionsX += tileTopLeft.x;
-					if (tilePositionsY !== '') {
-						tilePositionsY += ',';
-					}
-					tilePositionsY += tileTopLeft.y;
-					if (oldWireIds !== '') {
-						oldWireIds += ',';
-					}
-					if (this._tiles[key].oldWireId === undefined) {
-						oldWireIds += '0';
-					}
-					else {
-						oldWireIds += this._tiles[key].oldWireId;
-					}
 					needsNewTiles = true;
 					if (this._debug) {
 						this._debugAddInvalidationData(this._tiles[key]);
@@ -304,22 +280,9 @@ L.CalcTileLayer = L.TileLayer.extend({
 			}
 		}
 
-		if (needsNewTiles && command.part === this._selectedPart)
+		if (needsNewTiles && command.part === this._selectedPart && this._debug)
 		{
-			var message = 'tilecombine ' +
-				'part=' + command.part + ' ' +
-				'width=' + this._tileWidthPx + ' ' +
-				'height=' + this._tileHeightPx + ' ' +
-				'tileposx=' + tilePositionsX + ' ' +
-				'tileposy=' + tilePositionsY + ' ' +
-				'tilewidth=' + this._tileWidthTwips + ' ' +
-				'tileheight=' + this._tileHeightTwips + ' ' +
-				'oldwid=' + oldWireIds;
-
-			this._map._socket.sendMessage(message, '');
-			if (this._debug) {
-				this._debugAddInvalidationMessage(message);
-			}
+			this._debugAddInvalidationMessage(textMsg);
 		}
 
 		for (key in this._tileCache) {
@@ -359,11 +322,7 @@ L.CalcTileLayer = L.TileLayer.extend({
 	},
 
 	_onZoomRowColumns: function () {
-		this._updateClientZoom();
-		if (this._clientZoom) {
-			this._map._socket.sendMessage('clientzoom ' + this._clientZoom);
-			this._clientZoom = null;
-		}
+		this._sendClientZoom();
 		// TODO: test it!
 		this._map.fire('updaterowcolumnheaders');
 	},
