@@ -109,9 +109,6 @@ L.WriterTileLayer = L.TileLayer.extend({
 		var visibleTopLeft = this._latLngToTwips(this._map.getBounds().getNorthWest());
 		var visibleBottomRight = this._latLngToTwips(this._map.getBounds().getSouthEast());
 		var visibleArea = new L.Bounds(visibleTopLeft, visibleBottomRight);
-		var tilePositionsX = '';
-		var tilePositionsY = '';
-		var oldWireIds = '';
 		var needsNewTiles = false;
 		for (var key in this._tiles) {
 			var coords = this._tiles[key].coords;
@@ -126,23 +123,6 @@ L.WriterTileLayer = L.TileLayer.extend({
 					this._tiles[key]._invalidCount = 1;
 				}
 				if (visibleArea.intersects(bounds)) {
-					if (tilePositionsX !== '') {
-						tilePositionsX += ',';
-					}
-					tilePositionsX += tileTopLeft.x;
-					if (tilePositionsY !== '') {
-						tilePositionsY += ',';
-					}
-					tilePositionsY += tileTopLeft.y;
-					if (oldWireIds !== '') {
-						oldWireIds += ',';
-					}
-					if (this._tiles[key].oldWireId === undefined) {
-						oldWireIds += '0';
-					}
-					else {
-						oldWireIds += this._tiles[key].oldWireId;
-					}
 					needsNewTiles = true;
 					if (this._debug) {
 						this._debugAddInvalidationData(this._tiles[key]);
@@ -156,26 +136,9 @@ L.WriterTileLayer = L.TileLayer.extend({
 			}
 		}
 
-		if (needsNewTiles)
+		if (needsNewTiles && this._debug)
 		{
-			// CalcTileLayer.js and ImpressTileLayer.js avoid this when
-			// command.part !== this._selectedPart; but in Writer, the part is
-			// always 0 anyway
-			var message = 'tilecombine ' +
-				'part=' + command.part + ' ' +
-				'width=' + this._tileWidthPx + ' ' +
-				'height=' + this._tileHeightPx + ' ' +
-				'tileposx=' + tilePositionsX + ' ' +
-				'tileposy=' + tilePositionsY + ' ' +
-				'tilewidth=' + this._tileWidthTwips + ' ' +
-				'tileheight=' + this._tileHeightTwips + ' ' +
-				'oldwid=' + oldWireIds;
-
-			this._map._socket.sendMessage(message, '');
-
-			if (this._debug) {
-				this._debugAddInvalidationMessage(message);
-			}
+			this._debugAddInvalidationMessage(textMsg);
 		}
 
 		for (key in this._tileCache) {
