@@ -17,8 +17,11 @@ L.Map = L.Evented.extend({
 		trackResize: true,
 		markerZoomAnimation: true,
 		defaultZoom: 10,
-		tileWidthTwips: 3840,
-		tileHeightTwips: 3840,
+		// 15 = 1440 twips-per-inch / 96 dpi.
+		// Chosen to match previous hardcoded value of 3840 for
+		// the current tile pixel size of 256.
+		tileWidthTwips: window.tileSize * 15,
+		tileHeightTwips: window.tileSize * 15,
 		urlPrefix: 'lool',
 		wopiSrc: '',
 		cursorURL: 'images/cursors'
@@ -88,12 +91,12 @@ L.Map = L.Evented.extend({
 		this._socket = L.socket(this);
 		this._progressBar = L.progressOverlay(this.getCenter(), L.point(150, 25));
 
-		this._clipboardContainer = L.clipboardContainer();
-		this.addLayer(this._clipboardContainer);
-
-		// Inhibit the context menu - the browser thinks that the document
-		// is just a bunch of images, hence the context menu is useless (tdf#94599)
-		this.on('contextmenu', function() {});
+		if (L.Browser.mobile) {
+			this._clipboardContainer = L.control.mobileInput().addTo(this);
+		} else {
+			this._clipboardContainer = L.clipboardContainer();
+			this.addLayer(this._clipboardContainer);
+		}
 
 		// When all these conditions are met, fire statusindicator:initializationcomplete
 		this.initConditions = {
@@ -123,7 +126,7 @@ L.Map = L.Evented.extend({
 			if (!this.initComplete) {
 				this._fireInitComplete('doclayerinit');
 			}
-			if (this._docLayer._docType == 'text') {
+			if (!L.Browser.mobile && this._docLayer._docType == 'text') {
 				var interactiveRuler = this._permission === 'edit' ? true : false;
 				L.control.ruler({position:'topleft', interactive:interactiveRuler}).addTo(this);
 			}
@@ -270,7 +273,17 @@ L.Map = L.Evented.extend({
 			// we want it to be glued to the row/column headers instead of being centered
 			this._docLayer._checkSpreadSheetBounds(zoom);
 		}
-		return this.setView(this.getCenter(), zoom, {zoom: options});
+		var curCenter = this.getCenter();
+		if (this._docLayer && this._docLayer._visibleCursor && this.getBounds().contains(this._docLayer._visibleCursor.getCenter())) {
+			// Calculate new center after zoom. The intent is that the caret
+			// position stays the same.
+			var zoomScale = 1.0 / this.getZoomScale(zoom, this._zoom);
+			var caretPos = this._docLayer._visibleCursor.getCenter();
+			var newCenter = new L.LatLng(curCenter.lat + (caretPos.lat - curCenter.lat) * (1.0 - zoomScale),
+						     curCenter.lng + (caretPos.lng - curCenter.lng) * (1.0 - zoomScale));
+			return this.setView(newCenter, zoom, {zoom: options});
+		}
+		return this.setView(curCenter, zoom, {zoom: options});
 	},
 
 	zoomIn: function (delta, options) {
@@ -355,12 +368,41 @@ L.Map = L.Evented.extend({
 		return this.panTo(newCenter, options);
 	},
 
-	invalidateSize: function () {
+	invalidateSize: function (options) {
 		if (!this._loaded) { return this; }
 
+		options = L.extend({
+			animate: false,
+			pan: false
+		}, options === true ? {animate: true} : options);
+
 		var oldSize = this.getSize();
 		this._sizeChanged = true;
-		var newSize = this.getSize();
+
+		var newSize = this.getSize(),
+		    oldCenter = oldSize.divideBy(2).round(),
+		    newCenter = newSize.divideBy(2).round(),
+		    offset = oldCenter.subtract(newCenter);
+
+		if (!offset.x && !offset.y) { return this; }
+
+		if (options.animate && options.pan) {
+			this.panBy(offset);
+
+		} else {
+			if (options.pan) {
+				this._rawPanBy(offset);
+			}
+
+			this.fire('move');
+
+			if (options.debounceMoveend) {
+				clearTimeout(this._sizeTimer);
+				this._sizeTimer = setTimeout(L.bind(this.fire, this, 'moveend'), 200);
+			} else {
+				this.fire('moveend');
+			}
+		}
 
 		return this.fire('resize', {
 			oldSize: oldSize,
@@ -568,7 +610,7 @@ L.Map = L.Evented.extend({
 
 	getScaleZoom: function (scale, fromZoom) {
 		fromZoom = fromZoom === undefined ? this._zoom : fromZoom;
-		return fromZoom + (Math.log(scale) / Math.LN2);
+		return fromZoom + (Math.log(scale) / Math.log(1.2));
 	},
 
 
@@ -633,11 +675,7 @@ L.Map = L.Evented.extend({
 	},
 
 	focus: function () {
-		console.debug('focus:');
-		if (this._docLayer) {
-			console.debug('focus: focussing');
-			this._clipboardContainer.focus();
-		}
+		this._clipboardContainer.focus();
 	},
 
 	_fireInitComplete: function (condition) {
@@ -668,6 +706,10 @@ L.Map = L.Evented.extend({
 		this._fileDownloader = L.DomUtil.create('iframe', '', container);
 		L.DomUtil.setStyle(this._fileDownloader, 'display', 'none');
 
+		this._resizeDetector.contentWindow.addEventListener('touchstart', L.DomEvent.preventDefault, {passive: false});
+
+		L.DomEvent.on(this._resizeDetector.contentWindow, 'contextmenu', L.DomEvent.preventDefault);
+		L.DomEvent.on(this._fileDownloader.contentWindow, 'contextmenu', L.DomEvent.preventDefault);
 		L.DomEvent.addListener(container, 'scroll', this._onScroll, this);
 		container._leaflet = true;
 	},
@@ -788,7 +830,7 @@ L.Map = L.Evented.extend({
 		var onOff = remove ? 'off' : 'on';
 
 		L.DomEvent[onOff](this._container, 'click dblclick mousedown mouseup ' +
-			'mouseover mouseout mousemove contextmenu dragover drop ' +
+			'mouseover mouseout mousemove dragover drop ' +
 			'trplclick qdrplclick', this._handleDOMEvent, this);
 
 		if (this.options.trackResize && this._resizeDetector.contentWindow) {
@@ -802,7 +844,7 @@ L.Map = L.Evented.extend({
 	_onResize: function () {
 		L.Util.cancelAnimFrame(this._resizeRequest);
 		this._resizeRequest = L.Util.requestAnimFrame(
-		        function () { this.invalidateSize(); }, this, false, this._container);
+			function () { this.invalidateSize({debounceMoveend: true}); }, this, false, this._container);
 	},
 
 	_activate: function () {
@@ -820,7 +862,8 @@ L.Map = L.Evented.extend({
 				this._socket.sendMessage('useractive');
 				this._active = true;
 				if (this._docLayer) {
-					this._docLayer._onMessage('invalidatetiles: EMPTY', null);
+					this._docLayer._resetClientVisArea();
+					this._docLayer._requestNewTiles();
 				}
 
 				if (vex.dialogID > 0) {
@@ -960,15 +1003,9 @@ L.Map = L.Evented.extend({
 
 		console.debug('_onLostFocus: ');
 		var doclayer = this._docLayer;
-		if (!doclayer) { return; }
-
-		// save state of cursor (blinking marker) and the cursor overlay
-		doclayer._isCursorVisibleOnLostFocus = doclayer._isCursorVisible;
-		doclayer._isCursorOverlayVisibleOnLostFocus = doclayer._isCursorOverlayVisible;
-
-		// if the blinking cursor is visible, disable the overlay when we go out of focus
-		if (doclayer._isCursorVisible && doclayer._isCursorOverlayVisible) {
-			doclayer._isCursorOverlayVisible = false;
+		if (doclayer)
+		{
+			doclayer._isFocused = false;
 			doclayer._updateCursorAndOverlay();
 		}
 
@@ -980,16 +1017,14 @@ L.Map = L.Evented.extend({
 		if (!this._loaded) { return; }
 
 		var doclayer = this._docLayer;
-		if (doclayer &&
-		    typeof doclayer._isCursorOverlayVisibleOnLostFocus !== 'undefined' &&
-		    typeof doclayer._isCursorVisibleOnLostFocus !== 'undefined') {
+		if (doclayer)
+		{
+			doclayer._isFocused = true;
 			// we restore the old cursor position by a small delay, so that if the user clicks
 			// inside the document we skip to restore it, so that the user does not see the cursor
 			// jumping from the old position to the new one
 			setTimeout(function () {
-				// restore the state that was before focus was lost
-				doclayer._isCursorOverlayVisible = doclayer._isCursorOverlayVisibleOnLostFocus;
-				doclayer._isCursorVisible = doclayer._isCursorVisibleOnLostFocus;
+				console.debug('apply focus change in timeout');
 				doclayer._updateCursorAndOverlay();
 			}, 300);
 		}
@@ -1038,8 +1073,13 @@ L.Map = L.Evented.extend({
 
 		// For touch devices, to pop-up the keyboard, it is required to call
 		// .focus() method on hidden input within actual 'click' event here
-		// Calling from some other place with no real 'click' event doesn't work
-		if (type === 'click') {
+		// Calling from some other place with no real 'click' event doesn't work.
+
+		// (tml: For me, for this to work with a mobile device, we need to
+		// accept 'mouseup', too, and check the _wasSingleTap flag set over in Map.Tap.js.)
+		if (type === 'click' || (type === 'mouseup' &&
+					 typeof this._container._wasSingleTap !== 'undefined' &&
+					 this._container._wasSingleTap)) {
 			if (this._permission === 'edit') {
 				this.focus();
 			}
