@@ -10,7 +10,7 @@ var unoCommandsArray = {
 	BackgroundColor:{global:{menu:_('Background Color'),},},
 	Bold:{global:{menu:_('Bold'),},},
 	BringToFront:{global:{menu:_('~Bring to Front'),},},
-	CenterPara:{global:{context:_('Center Horizontally'),menu:_('Center'),},},
+	CenterPara:{global:{context:_('Align Center'),menu:_('Center'),},},
 	ChangeCaseRotateCase:{global:{menu:_('Cycle Case'),},},
 	ChangeCaseToLower:{global:{menu:_('~lowercase'),},},
 	ChangeCaseToSentenceCase:{global:{menu:_('~Sentence case'),},},
@@ -26,9 +26,10 @@ var unoCommandsArray = {
 	CommonAlignRight:{global:{menu:_('Right'),},},
 	CommonAlignTop:{global:{menu:_('Top'),},},
 	CommonAlignVerticalCenter:{global:{menu:_('Center'),},},
+	ConditionalFormatMenu:{spreadsheet:{menu:_('C~onditional'),},},
 	ContinueNumbering:{text:{menu:_('Continue previous numbering'),},},
 	ControlCodes:{text:{menu:_('For~matting Marks'),},},
-	Copy:{global:{menu:_('~Copy'),},},
+	Copy:{global:{menu:_('Cop~y'),},},
 	CopyHyperlinkLocation:{text:{menu:_('Copy Hyperlink Location'),},},
 	Cut:{global:{menu:_('~Cut'),},},
 	DataFilterAutoFilter:{spreadsheet:{menu:_('Auto~Filter'),},},
@@ -36,7 +37,7 @@ var unoCommandsArray = {
 	DataFilterRemoveFilter:{spreadsheet:{menu:_('~Reset Filter'),},},
 	DataFilterSpecialFilter:{spreadsheet:{menu:_('~Advanced Filter...'),},},
 	DataFilterStandardFilter:{spreadsheet:{menu:_('~Standard Filter...'),},},
-	DataMenu:{spreadsheet:{menu:_('~Data'),},},
+	DataMenu:{global:{menu:_('~Data'),},},
 	DataSort:{spreadsheet:{menu:_('~Sort...'),},},
 	DecrementIndent:{global:{context:_('Decrease Indent'),menu:_('Decrease Indent'),},},
 	DecrementLevel:{text:{menu:_('Demote One Level'),},},
@@ -47,10 +48,10 @@ var unoCommandsArray = {
 	DeleteAllNotes:{global:{menu:_('Delete All Comments'),},spreadsheet:{menu:_('Delete All Comments'),},},
 	DeleteAuthor:{global:{menu:_('Delete All Comments by This Author'),},},
 	DeleteColumnbreak:{spreadsheet:{menu:_('~Column Break'),},},
-	DeleteColumns:{presentation:{context:_('Delete Column'),menu:_('~Columns'),},spreadsheet:{menu:_('Delete Columns'),},text:{menu:_('~Columns'),},},
+	DeleteColumns:{presentation:{menu:_('Delete Column'),},spreadsheet:{menu:_('Delete Columns'),},text:{menu:_('~Columns'),},},
 	DeleteComment:{global:{menu:_('Delete Comment'),},},
 	DeleteRowbreak:{spreadsheet:{menu:_('~Row Break'),},},
-	DeleteRows:{presentation:{context:_('Delete Row'),menu:_('~Rows'),},spreadsheet:{menu:_('Delete Rows'),},text:{menu:_('~Rows'),},},
+	DeleteRows:{presentation:{menu:_('Delete Row'),},spreadsheet:{menu:_('Delete Rows'),},text:{menu:_('~Rows'),},},
 	DeleteSlide:{presentation:{menu:_('~Delete Slide'),},},
 	DeleteTable:{text:{menu:_('~Table'),},},
 	DownSearch:{global:{menu:_('Find Next'),},},
@@ -58,8 +59,8 @@ var unoCommandsArray = {
 	EditMenu:{global:{menu:_('~Edit'),},},
 	EditStyle:{global:{menu:_('~Edit Style...'),},presentation:{menu:_('E~dit Style...'),},},
 	EntireCell:{text:{menu:_('C~ell'),},},
-	EntireColumn:{presentation:{menu:_('~Columns'),},text:{menu:_('~Column'),},},
-	EntireRow:{presentation:{menu:_('~Rows'),},text:{menu:_('~Row'),},},
+	EntireColumn:{presentation:{menu:_('Select Column'),},text:{menu:_('~Column'),},},
+	EntireRow:{presentation:{menu:_('Select Row'),},text:{menu:_('~Row'),},},
 	FilterMenu:{spreadsheet:{menu:_('More ~Filters'),},},
 	FontColor:{global:{menu:_('Font Color'),},text:{menu:_('Font Color'),},},
 	FontDialog:{global:{menu:_('C~haracter...'),},},
@@ -87,11 +88,11 @@ var unoCommandsArray = {
 	IncrementLevel:{text:{menu:_('Promote One Level'),},},
 	IncrementSubLevels:{text:{menu:_('Promote One Level With Subpoints'),},},
 	InsertAnnotation:{global:{context:_('Insert Comment'),menu:_('Comme~nt'),},presentation:{menu:_('Comme~nt'),},spreadsheet:{context:_('Insert Co~mment'),menu:_('Comm~ent'),},},
-	InsertAuthorField:{presentation:{menu:_('~Author'),},text:{menu:_('~Author'),},},
+	InsertAuthorField:{presentation:{menu:_('~Author'),},text:{menu:_('First ~Author'),},},
 	InsertBreakMenu:{spreadsheet:{menu:_('Insert Page ~Break'),},},
 	InsertColumnBreak:{spreadsheet:{menu:_('~Column Break'),},text:{menu:_('Insert Column Break'),},},
-	InsertColumnsAfter:{presentation:{menu:_('Insert Column Right'),},spreadsheet:{context:_('Insert Columns ~Right'),menu:_('Columns ~Right'),},text:{menu:_('Columns R~ight'),},},
-	InsertColumnsBefore:{presentation:{menu:_('Insert Column Left'),},spreadsheet:{context:_('Insert Columns ~Left'),menu:_('Columns ~Left'),},text:{menu:_('Columns ~Left'),},},
+	InsertColumnsAfter:{presentation:{menu:_('Insert Column After'),},spreadsheet:{context:_('Insert Columns ~After'),menu:_('Columns ~After'),},text:{menu:_('Columns ~After'),},},
+	InsertColumnsBefore:{presentation:{menu:_('Insert Column Before'),},spreadsheet:{context:_('Insert Columns ~Before'),menu:_('Columns ~Before'),},text:{menu:_('Columns ~Before'),},},
 	InsertColumnsMenu:{spreadsheet:{menu:_('Insert Co~lumns'),},},
 	InsertDateField:{text:{menu:_('~Date'),},},
 	InsertEndnote:{text:{menu:_('~Endnote'),},},
@@ -182,7 +183,7 @@ var unoCommandsArray = {
 	SaveAs:{global:{menu:_('Save ~As...'),},},
 	SearchDialog:{global:{menu:_('Find & Rep~lace...'),},},
 	SelectAll:{global:{menu:_('Select ~All'),},},
-	SelectTable:{presentation:{menu:_('~Table'),},text:{menu:_('~Table'),},},
+	SelectTable:{presentation:{menu:_('~Select...'),},text:{menu:_('~Table'),},},
 	SendToBack:{global:{menu:_('~Send to Back'),},},
 	SetAnchorAtChar:{text:{menu:_('To ~Character'),},},
 	SetAnchorToChar:{text:{menu:_('As C~haracter'),},},
@@ -193,8 +194,8 @@ var unoCommandsArray = {
 	SetLanguageAllTextMenu:{global:{menu:_('For All Text'),},},
 	SetLanguageParagraphMenu:{global:{menu:_('For Paragraph'),},},
 	SetLanguageSelectionMenu:{global:{menu:_('For Selection'),},},
-	SetOptimalColumnWidth:{spreadsheet:{menu:_('~Optimal Width...'),},text:{menu:_('Optimal Column Width'),},},
-	SetOptimalRowHeight:{spreadsheet:{menu:_('~Optimal Height...'),},text:{menu:_('Optimal Row Height'),},},
+	SetOptimalColumnWidth:{presentation:{menu:_('Optimal Column Width'),},spreadsheet:{menu:_('~Optimal Width...'),},text:{menu:_('Optimal Column Width'),},},
+	SetOptimalRowHeight:{presentation:{menu:_('Optimal Row Height'),},spreadsheet:{menu:_('~Optimal Height...'),},text:{menu:_('Optimal Row Height'),},},
 	Shadowed:{global:{menu:_('Shadow'),},},
 	SheetMenu:{spreadsheet:{menu:_('~Sheet'),},},
 	Show:{spreadsheet:{menu:_('~Show Sheet...'),},},
@@ -216,10 +217,10 @@ var unoCommandsArray = {
 	Strikeout:{global:{menu:_('Strikethrough'),},},
 	SubScript:{global:{menu:_('Subscript'),},text:{menu:_('Subscript'),},},
 	SuperScript:{global:{menu:_('Superscript'),},text:{menu:_('Superscript'),},},
-	TableDeleteMenu:{text:{menu:_('~Delete'),},},
-	TableDialog:{presentation:{menu:_('Ta~ble Properties...'),},text:{menu:_('~Properties...'),},},
-	TableInsertMenu:{text:{menu:_('~Insert'),},},
-	TableMenu:{text:{menu:_('T~able'),},},
+	TableDeleteMenu:{global:{menu:_('~Delete'),},},
+	TableDialog:{presentation:{menu:_('~Properties...'),},text:{menu:_('~Properties...'),},},
+	TableInsertMenu:{global:{menu:_('~Insert'),},},
+	TableMenu:{global:{menu:_('T~able'),},},
 	TableSelectMenu:{text:{menu:_('~Select'),},},
 	TaskPaneInsertPage:{presentation:{menu:_('Insert Slide'),},},
 	TextAlign:{global:{menu:_('Alig~n'),},},
@@ -229,14 +230,14 @@ var unoCommandsArray = {
 	TrackChanges:{text:{menu:_('~Record'),},},
 	TransformDialog:{global:{menu:_('Position and Si~ze...'),},},
 	Underline:{global:{menu:_('Underline'),},},
-	UnderlineDouble:{presentation:{menu:_('Double Underline '),},spreadsheet:{menu:_('Underline: Double'),},text:{menu:_('Double Underline '),},},
-	Undo:{global:{menu:_('Undo'),},},
+	UnderlineDouble:{global:{menu:_('Double Underline'),},},
+	Undo:{global:{menu:_('~Undo'),},},
 	Ungroup:{global:{menu:_('~Ungroup...'),},},
 	UpSearch:{global:{menu:_('Find Previous'),},},
 	UpdateCurIndex:{text:{context:_('Update index'),menu:_('Current ~Index'),},},
 	Validation:{spreadsheet:{menu:_('~Validity...'),},},
 	ViewMenu:{global:{menu:_('~View'),},},
-	WordCountDialog:{text:{menu:_('~Word Count'),},},
+	WordCountDialog:{text:{menu:_('~Word Count...'),},},
 	WrapAnchorOnly:{text:{menu:_('~First Paragraph'),},},
 	WrapContour:{text:{menu:_('~Contour'),},},
 	WrapIdeal:{text:{menu:_('~Optimal Page Wrap'),},},
