@@ -22,6 +22,7 @@
 #include <cassert>
 #include <cerrno>
 #include <chrono>
+#include <climits>
 #include <cstdlib>
 #include <cstring>
 #include <fstream>
@@ -31,12 +32,13 @@
 #include <mutex>
 #include <sstream>
 #include <thread>
-#include <atomic>
 
-#include "common/Common.hpp"
-#include "common/Log.hpp"
-#include "common/Util.hpp"
-#include "common/SigUtil.hpp"
+#include "Common.hpp"
+#include "FakeSocket.hpp"
+#include "Log.hpp"
+#include "Util.hpp"
+#include "Protocol.hpp"
+#include "SigUtil.hpp"
 
 namespace Poco
 {
@@ -98,7 +100,6 @@ public:
     static const int DefaultSendBufferSize = 16 * 1024;
     static const int MaximumSendBufferSize = 128 * 1024;
     static std::atomic<bool> InhibitThreadChecks;
-    std::string _clientAddress;
 
     enum Type { IPv4, IPv6, All };
 
@@ -115,12 +116,26 @@ public:
         LOG_TRC("#" << getFD() << " Socket dtor.");
 
         // Doesn't block on sockets; no error handling needed.
-        close(_fd);
+#ifndef MOBILEAPP
+        ::close(_fd);
+#else
+        fakeSocketClose(_fd);
+#endif
     }
 
     /// Create socket of the given type.
     static int createSocket(Type type);
 
+    void setClientAddress(const std::string& address)
+    {
+        _clientAddress = address;
+    }
+
+    const std::string& clientAddress() const
+    {
+        return _clientAddress;
+    }
+
     /// Returns the OS native socket fd.
     int getFD() const { return _fd; }
 
@@ -129,7 +144,11 @@ public:
     virtual void shutdown()
     {
         LOG_TRC("#" << _fd << ": socket shutdown RDWR.");
+#ifndef MOBILEAPP
         ::shutdown(_fd, SHUT_RDWR);
+#else
+        fakeSocketShutdown(_fd);
+#endif
     }
 
     /// Prepare our poll record; adjust @timeoutMaxMs downwards
@@ -146,11 +165,14 @@ public:
     /// manage latency issues around packet aggregation
     void setNoDelay()
     {
+#ifndef MOBILEAPP
         const int val = 1;
         ::setsockopt(_fd, IPPROTO_TCP, TCP_NODELAY,
                      (char *) &val, sizeof(val));
+#endif
     }
 
+#ifndef MOBILEAPP
     /// Sets the kernel socket send buffer in size bytes.
     /// Note: TCP will allocate twice this size for admin purposes,
     /// so a subsequent call to getSendBufferSize will return
@@ -191,13 +213,19 @@ public:
         const int rc = ::getsockopt(_fd, SOL_SOCKET, SO_SNDBUF, &size, &len);
         return rc == 0 ? size : -1;
     }
+#endif
 
     /// Gets our fast cache of the socket buffer size
     int getSendBufferSize() const
     {
+#ifndef MOBILEAPP
         return _sendBufferSize;
+#else
+        return INT_MAX; // We want to always send a single record in one go
+#endif
     }
 
+#ifndef MOBILEAPP
     /// Sets the receive buffer size in bytes.
     /// Note: TCP will allocate twice this size for admin purposes,
     /// so a subsequent call to getReceieveBufferSize will return
@@ -238,6 +266,7 @@ public:
 
         return rc;
     }
+#endif
 
     virtual void dumpState(std::ostream&) {}
 
@@ -262,8 +291,8 @@ public:
     {
         if (InhibitThreadChecks)
             return;
-        // 0 owner means detached and can be invoked by any thread.
-        const bool sameThread = (_owner == std::thread::id(0) || std::this_thread::get_id() == _owner);
+        // uninitialized owner means detached and can be invoked by any thread.
+        const bool sameThread = (_owner == std::thread::id() || std::this_thread::get_id() == _owner);
         if (!sameThread)
             LOG_ERR("#" << _fd << " Invoked from foreign thread. Expected: " <<
                     Log::to_string(_owner) << " but called from " <<
@@ -289,6 +318,7 @@ protected:
         _owner = std::this_thread::get_id();
         LOG_DBG("#" << _fd << " Thread affinity set to " << Log::to_string(_owner) << ".");
 
+#ifndef MOBILEAPP
 #if ENABLE_DEBUG
         if (std::getenv("LOOL_ZERO_BUFFER_SIZE"))
         {
@@ -297,10 +327,12 @@ protected:
             LOG_TRC("#" << _fd << ": Buffer size: " << getSendBufferSize() <<
                     " (was " << oldSize << ")");
         }
+#endif
 #endif
     }
 
 private:
+    std::string _clientAddress;
     const int _fd;
     int _sendBufferSize;
 
@@ -369,6 +401,18 @@ public:
     {
         LOG_DBG("Stopping " << _name << ".");
         _stop = true;
+#ifdef MOBILEAPP
+        {
+            // We don't want to risk some callbacks in _newCallbacks being invoked when we start
+            // running a thread for this SocketPoll again.
+            std::lock_guard<std::mutex> lock(_mutex);
+            if (_newCallbacks.size() > 0)
+            {
+                LOG_TRC("_newCallbacks is non-empty, clearing it");
+                _newCallbacks.clear();
+            }
+        }
+#endif
         wakeup();
     }
 
@@ -384,13 +428,13 @@ public:
 
             LOG_DBG("Removing socket #" << socket->getFD() << " from " << _name);
             socket->assertCorrectThread();
-            socket->setThreadOwner(std::thread::id(0));
+            socket->setThreadOwner(std::thread::id());
 
             _pollSockets.pop_back();
         }
     }
 
-    bool isAlive() const { return _threadStarted && !_threadFinished; }
+    bool isAlive() const { return (_threadStarted && !_threadFinished) || _runOnClientThread; }
 
     /// Check if we should continue polling
     virtual bool continuePolling()
@@ -416,8 +460,8 @@ public:
     {
         if (InhibitThreadChecks)
             return;
-        // 0 owner means detached and can be invoked by any thread.
-        const bool sameThread = (!isAlive() || _owner == std::thread::id(0) || std::this_thread::get_id() == _owner);
+        // uninitialized owner means detached and can be invoked by any thread.
+        const bool sameThread = (!isAlive() || _owner == std::thread::id() || std::this_thread::get_id() == _owner);
         if (!sameThread)
             LOG_ERR("Incorrect thread affinity for " << _name << ". Expected: " <<
                     Log::to_string(_owner) << " (" << Util::getThreadId() <<
@@ -441,7 +485,12 @@ public:
         int rc;
         do
         {
+#ifndef MOBILEAPP
             rc = ::poll(&_pollFds[0], size + 1, std::max(timeoutMaxMs,0));
+#else
+            LOG_TRC("SocketPoll Poll");
+            rc = fakeSocketPoll(&_pollFds[0], size + 1, std::max(timeoutMaxMs,0));
+#endif
         }
         while (rc < 0 && errno == EINTR);
         LOG_TRC("Poll completed with " << rc << " live polls max (" <<
@@ -455,8 +504,12 @@ public:
                 std::lock_guard<std::mutex> lock(_mutex);
 
                 // Clear the data.
+#ifndef MOBILEAPP
                 int dump = ::read(_wakeup[0], &dump, sizeof(dump));
-
+#else
+                LOG_TRC("Wakeup pipe read");
+                int dump = fakeSocketRead(_wakeup[0], &dump, sizeof(dump));
+#endif
                 // Copy the new sockets over and clear.
                 _pollSockets.insert(_pollSockets.end(),
                                     _newSockets.begin(), _newSockets.end());
@@ -535,7 +588,26 @@ public:
         // wakeup the main-loop.
         int rc;
         do {
+#ifndef MOBILEAPP
             rc = ::write(fd, "w", 1);
+#else
+#if 0
+            // Our fake sockets are record-oriented with a single record buffer, so as we write one
+            // byte at a time to the wakeup pipe, we can't write another one before the previous one
+            // has been read.
+            //
+            // FIXME: Still, explicitly waiting here with fakeSocketPoll() for it to be writable
+            // doesn't seem to be any improvement. On the contrary, with this fakeSocketPoll(), we
+            // hang every time we run the app. Without it, we only hang occasionally.
+
+            LOG_TRC("Wakeup pipe write");
+            struct pollfd p;
+            p.fd = fd;
+            p.events = POLLOUT;
+            fakeSocketPoll(&p, 1, -1);
+#endif
+            rc = fakeSocketWrite(fd, "w", 1);
+#endif
         } while (rc == -1 && errno == EINTR);
 
         if (rc == -1 && errno != EAGAIN && errno != EWOULDBLOCK)
@@ -564,7 +636,7 @@ public:
             std::lock_guard<std::mutex> lock(_mutex);
             LOG_DBG("Inserting socket #" << newSocket->getFD() << " into " << _name);
             // sockets in transit are un-owned.
-            newSocket->setThreadOwner(std::thread::id(0));
+            newSocket->setThreadOwner(std::thread::id());
             _newSockets.emplace_back(newSocket);
             wakeup();
         }
@@ -572,12 +644,18 @@ public:
 
     /// Inserts a new websocket to be polled.
     /// NOTE: The DNS lookup is synchronous.
-    void insertNewWebSocketSync(const Poco::URI &uri, const std::shared_ptr<SocketHandlerInterface>& websocketHandler);
+    void insertNewWebSocketSync(
+#ifndef MOBILEAPP
+                                const Poco::URI &uri,
+#else
+                                int peerSocket,
+#endif
+                                const std::shared_ptr<SocketHandlerInterface>& websocketHandler);
 
     typedef std::function<void()> CallbackFn;
 
     /// Add a callback to be invoked in the polling thread
-    void addCallback(CallbackFn fn)
+    void addCallback(const CallbackFn& fn)
     {
         std::lock_guard<std::mutex> lock(_mutex);
         _newCallbacks.emplace_back(fn);
@@ -611,11 +689,34 @@ public:
     const std::string& name() const { return _name; }
 
     /// Start the polling thread (if desired)
-    void startThread();
+    /// Mutually exclusive with runOnClientThread().
+    bool startThread();
 
     /// Stop and join the polling thread before returning (if active)
     void joinThread();
 
+    /// Called to prevent starting own poll thread
+    /// when polling is done on the client's thread.
+    /// Mutually exclusive with startThread().
+    bool runOnClientThread()
+    {
+        assert(!_threadStarted);
+
+        if (!_threadStarted)
+        {
+            _runOnClientThread = true;
+            return true;
+        }
+
+        return false;
+    }
+
+protected:
+    bool isStop() const
+    {
+        return _stop;
+    }
+
 private:
     /// Initialize the poll fds array with the right events
     void setupPollFds(std::chrono::steady_clock::time_point now,
@@ -628,16 +729,9 @@ private:
         for (size_t i = 0; i < size; ++i)
         {
             int events = _pollSockets[i]->getPollEvents(now, timeoutMaxMs);
-            if (events < 0) // timeout on dead socket
-            {
-                _pollFds[i].fd = _wakeup[0];
-                _pollFds[i].events = 0;
-            }
-            else
-            {
-                _pollFds[i].fd = _pollSockets[i]->getFD();
-                _pollFds[i].events = events;
-            }
+            assert(events >= 0); // Or > 0 even?
+            _pollFds[i].fd = _pollSockets[i]->getFD();
+            _pollFds[i].events = events;
             _pollFds[i].revents = 0;
         }
 
@@ -649,33 +743,8 @@ private:
 
     /// The polling thread entry.
     /// Used to set the thread name and mark the thread as stopped when done.
-    void pollingThreadEntry()
-    {
-        try
-        {
-            Util::setThreadName(_name);
-            LOG_INF("Starting polling thread [" << _name << "].");
+    void pollingThreadEntry();
 
-            _owner = std::this_thread::get_id();
-            LOG_DBG("Thread affinity of " << _name << " set to " <<
-                    Log::to_string(_owner) << ".");
-
-            // Invoke the virtual implementation.
-            pollingThread();
-
-            // Release sockets.
-            _pollSockets.clear();
-            _newSockets.clear();
-        }
-        catch (const std::exception& exc)
-        {
-            LOG_ERR("Exception in polling thread [" << _name << "]: " << exc.what());
-        }
-
-        _threadFinished = true;
-    }
-
-private:
     /// Debug name used for logging.
     const std::string _name;
 
@@ -683,20 +752,20 @@ private:
     int _wakeup[2];
     /// The sockets we're controlling
     std::vector<std::shared_ptr<Socket>> _pollSockets;
-    /// Protects _newSockets
+    /// Protects _newSockets and _newCallbacks
     std::mutex _mutex;
     std::vector<std::shared_ptr<Socket>> _newSockets;
     std::vector<CallbackFn> _newCallbacks;
     /// The fds to poll.
     std::vector<pollfd> _pollFds;
 
-protected:
     /// Flag the thread to stop.
     std::atomic<bool> _stop;
     /// The polling thread.
     std::thread _thread;
     std::atomic<bool> _threadStarted;
     std::atomic<bool> _threadFinished;
+    std::atomic<bool> _runOnClientThread;
     std::thread::id _owner;
 };
 
@@ -795,6 +864,7 @@ public:
     {
         assertCorrectThread();
 
+#ifndef MOBILEAPP
         // SSL decodes blocks of 16Kb, so for efficiency we use the same.
         char buf[16 * 1024];
         ssize_t len;
@@ -817,6 +887,24 @@ public:
             // else poll will handle errors.
         }
         while (len == (sizeof(buf)));
+#else
+        LOG_TRC("readIncomingData #" << getFD());
+        ssize_t available = fakeSocketAvailableDataLength(getFD());
+        ssize_t len;
+        if (available == -1)
+            len = -1;
+        else if (available == 0)
+            len = 0;
+        else
+        {
+            std::vector<char>buf(available);
+            len = readData(buf.data(), available);
+            assert(len == available);
+            _bytesRecvd += len;
+            assert(_inBuffer.size() == 0);
+            _inBuffer.insert(_inBuffer.end(), buf.data(), buf.data() + len);
+        }
+#endif
 
         return len != 0; // zero is eof / clean socket close.
     }
@@ -844,11 +932,7 @@ public:
     /// Remove the first @count bytes from input buffer
     void eraseFirstInputBytes(size_t count)
     {
-        size_t toErase = std::min(count, _inBuffer.size());
-        if (toErase < count)
-            LOG_ERR("#" << getFD() << ": attempted to remove: " << count << " which is > size: " << _inBuffer.size() << " clamped to " << toErase);
-        if (toErase > 0)
-            _inBuffer.erase(_inBuffer.begin(), _inBuffer.begin() + count);
+        _inBuffer.erase(_inBuffer.begin(), _inBuffer.begin() + count);
     }
 
     /// Detects if we have an HTTP header in the provided message and
@@ -865,9 +949,14 @@ public:
         recv = _bytesRecvd;
     }
 
-    const std::string clientAddress()
+    std::vector<char>& getInBuffer()
     {
-        return _clientAddress;
+        return _inBuffer;
+    }
+
+    std::vector<char>& getOutBuffer()
+    {
+        return _outBuffer;
     }
 
 protected:
@@ -944,6 +1033,7 @@ protected:
             disposition.setClosed();
     }
 
+public:
     /// Override to write data out to socket.
     virtual void writeOutgoingData()
     {
@@ -983,23 +1073,54 @@ protected:
         while (!_outBuffer.empty());
     }
 
+    /// Does it look like we have some TLS / SSL where we don't expect it ?
+    bool sniffSSL() const;
+
+protected:
     /// Override to handle reading of socket data differently.
     virtual int readData(char* buf, int len)
     {
         assertCorrectThread();
+#ifndef MOBILEAPP
         return ::read(getFD(), buf, len);
+#else
+        return fakeSocketRead(getFD(), buf, len);
+#endif
     }
 
     /// Override to handle writing data to socket differently.
     virtual int writeData(const char* buf, const int len)
     {
         assertCorrectThread();
+#ifndef MOBILEAPP
         return ::write(getFD(), buf, len);
+#else
+        struct pollfd p;
+        p.fd = getFD();
+        p.events = POLLOUT;
+        fakeSocketPoll(&p, 1, -1);
+        return fakeSocketWrite(getFD(), buf, len);
+#endif
     }
 
     void dumpState(std::ostream& os) override;
 
-protected:
+    void setShutdownSignalled(bool shutdownSignalled)
+    {
+        _shutdownSignalled = shutdownSignalled;
+    }
+
+    bool isShutdownSignalled() const
+    {
+        return _shutdownSignalled;
+    }
+
+    const std::shared_ptr<SocketHandlerInterface>& getSocketHandler() const
+    {
+        return _socketHandler;
+    }
+
+  private:
     /// Client handling the actual data.
     std::shared_ptr<SocketHandlerInterface> _socketHandler;
 
@@ -1016,14 +1137,6 @@ protected:
 
     /// True when shutdown was requested via shutdown().
     bool _shutdownSignalled;
-
-    // To be able to access _inBuffer and _outBuffer.
-    // TODO we probably need accessors to the _inBuffer & _outBuffer
-    // instead of this many friends...
-    friend class WebSocketHandler;
-    friend class ClientRequestDispatcher;
-    friend class PrisonerRequestDispatcher;
-    friend class SimpleResponseClient;
 };
 
 enum class WSOpCode : unsigned char {
