@@ -7,14 +7,14 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <assert.h>
 #include "Ssl.hpp"
 
 #include <sys/syscall.h>
 
-#include "Util.hpp"
+#include <Util.hpp>
 
 extern "C"
 {
@@ -66,9 +66,9 @@ SslContext::SslContext(const std::string& certFilePath,
     // Create the Context. We only have one,
     // as we don't expect/support different servers in same process.
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
-    _ctx = SSL_CTX_new(TLS_server_method());
+    _ctx = SSL_CTX_new(TLS_method());
 #else
-    _ctx = SSL_CTX_new(SSLv23_server_method());
+    _ctx = SSL_CTX_new(SSLv23_method());
 #endif
 
     // SSL_CTX_set_default_passwd_cb(_ctx, &privateKeyPassphraseCallback);
@@ -80,7 +80,7 @@ SslContext::SslContext(const std::string& certFilePath,
         int errCode = 0;
         if (!caFilePath.empty())
         {
-            errCode = SSL_CTX_load_verify_locations(_ctx, caFilePath.c_str(), 0);
+            errCode = SSL_CTX_load_verify_locations(_ctx, caFilePath.c_str(), nullptr);
             if (errCode != 1)
             {
                 std::string msg = getLastErrorMsg();
@@ -248,9 +248,9 @@ void SslContext::initDH()
     // OpenSSL v1.1.0 has public API changes
     // p, g and length of the Diffie-Hellman param can't be set directly anymore,
     // instead DH_set0_pqg and DH_set_length are used
-    BIGNUM* p = BN_bin2bn(dh1024_p, sizeof(dh1024_p), 0);
-    BIGNUM* g = BN_bin2bn(dh1024_g, sizeof(dh1024_g), 0);
-    if ((DH_set0_pqg(dh, p, NULL, g) == 0) || (DH_set_length(dh, 160) == 0))
+    BIGNUM* p = BN_bin2bn(dh1024_p, sizeof(dh1024_p), nullptr);
+    BIGNUM* g = BN_bin2bn(dh1024_g, sizeof(dh1024_g), nullptr);
+    if ((DH_set0_pqg(dh, p, nullptr, g) == 0) || (DH_set_length(dh, 160) == 0))
 #else
     dh->p = BN_bin2bn(dh1024_p, sizeof(dh1024_p), 0);
     dh->g = BN_bin2bn(dh1024_g, sizeof(dh1024_g), 0);
