@@ -18,6 +18,7 @@
 #include <Poco/Util/Application.h>
 
 #include "Auth.hpp"
+#include "LOOLWSD.hpp"
 #include "Log.hpp"
 #include "Util.hpp"
 
@@ -121,7 +122,7 @@ public:
         _isUserModified(false),
         _isAutosave(false)
     {
-        LOG_DBG("Storage ctor: " << uri.toString());
+        LOG_DBG("Storage ctor: " << LOOLWSD::anonymizeUrl(uri.toString()));
     }
 
     virtual ~StorageBase() {}
@@ -150,6 +151,8 @@ public:
 
     /// To be able to set the WOPI 'is autosave?' header appropriately.
     void setIsAutosave(bool isAutosave) { _isAutosave = isAutosave; }
+    void setIsExitSave(bool isExitSave) { _isExitSave = isExitSave; }
+    bool isExitSave() const { return _isExitSave; }
 
     /// Returns the basic information about the file.
     const FileInfo& getFileInfo() const { return _fileInfo; }
@@ -173,6 +176,8 @@ public:
     static std::unique_ptr<StorageBase> create(const Poco::URI& uri,
                                                const std::string& jailRoot,
                                                const std::string& jailPath);
+
+    static bool allowedWopiHost(const std::string& host);
 protected:
 
     /// Returns the root path of the jail directory of docs.
@@ -183,6 +188,7 @@ protected:
     std::string _localStorePath;
     std::string _jailPath;
     std::string _jailedFilePath;
+    std::string _jailedFilePathAnonym;
     FileInfo _fileInfo;
     bool _isLoaded;
     bool _forceSave;
@@ -192,6 +198,8 @@ protected:
 
     /// This save operation is an autosave.
     bool _isAutosave;
+    /// Saving on exit (when the document is cleaned up from memory)
+    bool _isExitSave;
 
     static bool FilesystemEnabled;
     static bool WopiEnabled;
@@ -210,20 +218,20 @@ public:
         _isCopy(false)
     {
         LOG_INF("LocalStorage ctor with localStorePath: [" << localStorePath <<
-                "], jailPath: [" << jailPath << "], uri: [" << uri.toString() << "].");
+                "], jailPath: [" << jailPath << "], uri: [" << LOOLWSD::anonymizeUrl(uri.toString()) << "].");
     }
 
     class LocalFileInfo
     {
     public:
-        LocalFileInfo(const std::string& userid,
+        LocalFileInfo(const std::string& userId,
                       const std::string& username)
-            : _userid(userid),
+            : _userId(userId),
               _username(username)
         {
         }
 
-        std::string _userid;
+        std::string _userId;
         std::string _username;
     };
 
@@ -253,13 +261,21 @@ public:
         _wopiLoadDuration(0)
     {
         LOG_INF("WopiStorage ctor with localStorePath: [" << localStorePath <<
-                "], jailPath: [" << jailPath << "], uri: [" << uri.toString() << "].");
+                "], jailPath: [" << jailPath << "], uri: [" << LOOLWSD::anonymizeUrl(uri.toString()) << "].");
     }
 
     class WOPIFileInfo
     {
     public:
+        enum class TriState
+        {
+            False,
+            True,
+            Unset
+        };
+
         WOPIFileInfo(const std::string& userid,
+                     const std::string& obfuscatedUserId,
                      const std::string& username,
                      const std::string& userExtraInfo,
                      const std::string& watermarkText,
@@ -268,15 +284,20 @@ public:
                      const bool hidePrintOption,
                      const bool hideSaveOption,
                      const bool hideExportOption,
-                     const bool hideChangeTrackingControls,
                      const bool enableOwnerTermination,
                      const bool disablePrint,
                      const bool disableExport,
                      const bool disableCopy,
                      const bool disableInactiveMessages,
                      const bool userCanNotWriteRelative,
+                     const bool enableInsertRemoteImage,
+                     const bool enableShare,
+                     const TriState disableChangeTrackingShow,
+                     const TriState disableChangeTrackingRecord,
+                     const TriState hideChangeTrackingControls,
                      const std::chrono::duration<double> callDuration)
-            : _userid(userid),
+            : _userId(userid),
+              _obfuscatedUserId(obfuscatedUserId),
               _username(username),
               _watermarkText(watermarkText),
               _userCanWrite(userCanWrite),
@@ -284,20 +305,26 @@ public:
               _hidePrintOption(hidePrintOption),
               _hideSaveOption(hideSaveOption),
               _hideExportOption(hideExportOption),
-              _hideChangeTrackingControls(hideChangeTrackingControls),
               _enableOwnerTermination(enableOwnerTermination),
               _disablePrint(disablePrint),
               _disableExport(disableExport),
               _disableCopy(disableCopy),
               _disableInactiveMessages(disableInactiveMessages),
               _userCanNotWriteRelative(userCanNotWriteRelative),
+              _enableInsertRemoteImage(enableInsertRemoteImage),
+              _enableShare(enableShare),
+              _disableChangeTrackingShow(disableChangeTrackingShow),
+              _disableChangeTrackingRecord(disableChangeTrackingRecord),
+              _hideChangeTrackingControls(hideChangeTrackingControls),
               _callDuration(callDuration)
             {
                 _userExtraInfo = userExtraInfo;
             }
 
         /// User id of the user accessing the file
-        std::string _userid;
+        std::string _userId;
+        /// Obfuscated User id used for logging the UserId.
+        std::string _obfuscatedUserId;
         /// Display Name of user accessing the file
         std::string _username;
         /// Extra info per user, typically mail and other links, as json.
@@ -314,8 +341,6 @@ public:
         bool _hideSaveOption;
         /// Hide 'Download as' button/menubar item from UI
         bool _hideExportOption;
-        /// Hide change tacking menu from UI
-        bool _hideChangeTrackingControls;
         /// If WOPI host has enabled owner termination feature on
         bool _enableOwnerTermination;
         /// If WOPI host has allowed the user to print the document
@@ -328,6 +353,17 @@ public:
         bool _disableInactiveMessages;
         /// If set to false, users can access the save-as functionality
         bool _userCanNotWriteRelative;
+        /// if set to true, users can access the insert remote image functionality
+        bool _enableInsertRemoteImage;
+        /// if set to true, users can access the file share functionality
+        bool _enableShare;
+        /// If we should disable change-tracking visibility by default (meaningful at loading).
+        TriState _disableChangeTrackingShow;
+        /// If we should disable change-tracking ability by default (meaningful at loading).
+        TriState _disableChangeTrackingRecord;
+        /// If we should hide change-tracking commands for this user.
+        TriState _hideChangeTrackingControls;
+
         /// Time it took to call WOPI's CheckFileInfo
         std::chrono::duration<double> _callDuration;
     };
@@ -363,7 +399,7 @@ public:
         _authAgent(std::move(authAgent))
     {
         LOG_INF("WebDAVStorage ctor with localStorePath: [" << localStorePath <<
-                "], jailPath: [" << jailPath << "], uri: [" << uri.toString() << "].");
+                "], jailPath: [" << jailPath << "], uri: [" << LOOLWSD::anonymizeUrl(uri.toString()) << "].");
     }
 
     // Implement me
