@@ -47,14 +47,18 @@ using Poco::Timestamp;
 
 TileCache::TileCache(const std::string& docURL,
                      const Timestamp& modifiedTime,
-                     const std::string& cacheDir) :
+                     const std::string& cacheDir,
+                     const bool tileCachePersistent) :
     _docURL(docURL),
-    _cacheDir(cacheDir)
+    _cacheDir(cacheDir),
+    _tileCachePersistent(tileCachePersistent)
 {
-    LOG_INF("TileCache ctor for uri [" << _docURL <<
+#ifndef BUILDING_TESTS
+    LOG_INF("TileCache ctor for uri [" << LOOLWSD::anonymizeUrl(_docURL) <<
             "], cacheDir: [" << _cacheDir <<
             "], modifiedTime=" << (modifiedTime.raw()/1000000) <<
             " getLastModified()=" << (getLastModified().raw()/1000000));
+#endif
     File directory(_cacheDir);
     std::string unsaved;
     if (directory.exists() &&
@@ -72,8 +76,10 @@ TileCache::TileCache(const std::string& docURL,
 
 TileCache::~TileCache()
 {
-    _owner = std::thread::id(0);
-    LOG_INF("~TileCache dtor for uri [" << _docURL << "].");
+    _owner = std::thread::id();
+#ifndef BUILDING_TESTS
+    LOG_INF("~TileCache dtor for uri [" << LOOLWSD::anonymizeUrl(_docURL) << "].");
+#endif
 }
 
 void TileCache::completeCleanup() const
@@ -87,8 +93,6 @@ void TileCache::completeCleanup() const
 /// rendering latency.
 struct TileCache::TileBeingRendered
 {
-    std::vector<std::weak_ptr<ClientSession>> _subscribers;
-
     TileBeingRendered(const std::string& cachedName, const TileDesc& tile)
      : _startTime(std::chrono::steady_clock::now()),
        _tile(tile),
@@ -104,7 +108,9 @@ struct TileCache::TileBeingRendered
     std::chrono::steady_clock::time_point getStartTime() const { return _startTime; }
     double getElapsedTimeMs() const { return std::chrono::duration_cast<std::chrono::milliseconds>
                                               (std::chrono::steady_clock::now() - _startTime).count(); }
+    std::vector<std::weak_ptr<ClientSession>>& getSubscribers() { return _subscribers; }
 private:
+    std::vector<std::weak_ptr<ClientSession>> _subscribers;
     std::chrono::steady_clock::time_point _startTime;
     TileDesc _tile;
     std::string _cachedName;
@@ -120,17 +126,50 @@ std::shared_ptr<TileCache::TileBeingRendered> TileCache::findTileBeingRendered(c
     return tile != _tilesBeingRendered.end() ? tile->second : nullptr;
 }
 
-void TileCache::forgetTileBeingRendered(const TileDesc& tile)
+void TileCache::forgetTileBeingRendered(const std::shared_ptr<TileCache::TileBeingRendered>& tileBeingRendered, const TileDesc& tile)
 {
-    const std::string cachedName = cacheFileName(tile);
-
     assertCorrectThread();
+    assert(tileBeingRendered);
+    assert(_tilesBeingRendered.find(tileBeingRendered->getCacheName()) != _tilesBeingRendered.end());
 
-    _tilesBeingRendered.erase(cachedName);
+    for(auto& subscriber : tileBeingRendered->getSubscribers())
+    {
+        std::shared_ptr<ClientSession> session = subscriber.lock();
+        if(session && tile.getId() == -1)
+            session->traceUnSubscribeToTile(tileBeingRendered->getCacheName());
+    }
+
+    _tilesBeingRendered.erase(tileBeingRendered->getCacheName());
+}
+
+double TileCache::getTileBeingRenderedElapsedTimeMs(const std::string& tileCacheName) const
+{
+    auto iterator = _tilesBeingRendered.find(tileCacheName);
+    if(iterator == _tilesBeingRendered.end())
+        return -1.0; // Negativ value means that we did not find tileBeingRendered object
+
+    return iterator->second->getElapsedTimeMs();
+}
+
+bool TileCache::hasTileBeingRendered(const TileDesc& tile)
+{
+    return (findTileBeingRendered(tile) != nullptr);
+}
+
+int TileCache::getTileBeingRenderedVersion(const TileDesc& tile)
+{
+    std::shared_ptr<TileBeingRendered> tileBeingRendered = findTileBeingRendered(tile);
+    if (tileBeingRendered)
+        return tileBeingRendered->getVersion();
+    else
+        return 0;
 }
 
 std::unique_ptr<std::fstream> TileCache::lookupTile(const TileDesc& tile)
 {
+    if (!_tileCachePersistent)
+        return nullptr;
+
     const std::string fileName = _cacheDir + "/" + cacheFileName(tile);
 
     std::unique_ptr<std::fstream> result(new std::fstream(fileName, std::ios::in));
@@ -160,7 +199,7 @@ void TileCache::saveTileAndNotify(const TileDesc& tile, const char *data, const
     // Ignore if we can't save the tile, things will work anyway, but slower.
     // An error indication is supposed to be sent to all users in that case.
     const auto fileName = _cacheDir + "/" + cachedName;
-    if (FileUtil::saveDataToFileSafely(fileName, data, size))
+    if (_tileCachePersistent && FileUtil::saveDataToFileSafely(fileName, data, size))
     {
         LOG_TRC("Saved cache tile: " << fileName);
     }
@@ -168,7 +207,7 @@ void TileCache::saveTileAndNotify(const TileDesc& tile, const char *data, const
     // Notify subscribers, if any.
     if (tileBeingRendered)
     {
-        const size_t subscriberCount = tileBeingRendered->_subscribers.size();
+        const size_t subscriberCount = tileBeingRendered->getSubscribers().size();
         if (subscriberCount > 0)
         {
             std::string response = tile.serialize("tile:");
@@ -181,10 +220,12 @@ void TileCache::saveTileAndNotify(const TileDesc& tile, const char *data, const
             payload->append("\n", 1);
             payload->append(data, size);
 
-            auto& firstSubscriber = tileBeingRendered->_subscribers[0];
+            auto& firstSubscriber = tileBeingRendered->getSubscribers()[0];
             std::shared_ptr<ClientSession> firstSession = firstSubscriber.lock();
             if (firstSession)
+            {
                 firstSession->enqueueSendMessage(payload);
+            }
 
             if (subscriberCount > 1)
             {
@@ -200,7 +241,7 @@ void TileCache::saveTileAndNotify(const TileDesc& tile, const char *data, const
 
                 for (size_t i = 1; i < subscriberCount; ++i)
                 {
-                    auto& subscriber = tileBeingRendered->_subscribers[i];
+                    auto& subscriber = tileBeingRendered->getSubscribers()[i];
                     std::shared_ptr<ClientSession> session = subscriber.lock();
                     if (session)
                     {
@@ -219,7 +260,7 @@ void TileCache::saveTileAndNotify(const TileDesc& tile, const char *data, const
         {
             LOG_DBG("STATISTICS: tile " << tile.getVersion() << " internal roundtrip " <<
                     tileBeingRendered->getElapsedTimeMs() << " ms.");
-            _tilesBeingRendered.erase(cachedName);
+            forgetTileBeingRendered(tileBeingRendered, tile);
         }
     }
     else
@@ -285,7 +326,7 @@ void TileCache::setUnsavedChanges(bool state)
         removeFile("unsaved.txt");
 }
 
-void TileCache::saveRendering(const std::string& name, const std::string& dir, const char *data, size_t size)
+void TileCache::saveRendering(const std::string& name, const std::string& dir, const char *data, std::size_t size)
 {
     // can fonts be invalidated?
     const std::string dirName = _cacheDir + "/" + dir;
@@ -338,6 +379,13 @@ void TileCache::invalidateTiles(int part, int x, int y, int width, int height)
 }
 
 void TileCache::invalidateTiles(const std::string& tiles)
+{
+    std::pair<int, Util::Rectangle> result = TileCache::parseInvalidateMsg(tiles);
+    Util::Rectangle& invalidateRect = result.second;
+    invalidateTiles(result.first, invalidateRect.getLeft(), invalidateRect.getTop(), invalidateRect.getWidth(), invalidateRect.getHeight());
+}
+
+std::pair<int, Util::Rectangle> TileCache::parseInvalidateMsg(const std::string& tiles)
 {
     StringTokenizer tokens(tiles, " ", StringTokenizer::TOK_IGNORE_EMPTY | StringTokenizer::TOK_TRIM);
 
@@ -345,16 +393,14 @@ void TileCache::invalidateTiles(const std::string& tiles)
 
     if (tokens.count() == 2 && tokens[1] == "EMPTY")
     {
-        invalidateTiles(-1, 0, 0, INT_MAX, INT_MAX);
-        return;
+        return std::pair<int, Util::Rectangle>(-1, Util::Rectangle(0, 0, INT_MAX, INT_MAX));
     }
     else if (tokens.count() == 3 && tokens[1] == "EMPTY,")
     {
         int part = 0;
         if (stringToInteger(tokens[2], part))
         {
-            invalidateTiles(part, 0, 0, INT_MAX, INT_MAX);
-            return;
+            return std::pair<int, Util::Rectangle>(part, Util::Rectangle(0, 0, INT_MAX, INT_MAX));
         }
     }
     else
@@ -367,12 +413,13 @@ void TileCache::invalidateTiles(const std::string& tiles)
             getTokenInteger(tokens[4], "width", width) &&
             getTokenInteger(tokens[5], "height", height))
         {
-            invalidateTiles(part, x, y, width, height);
-            return;
+
+            return std::pair<int, Util::Rectangle>(part, Util::Rectangle(x, y, width, height));
         }
     }
 
     LOG_ERR("Unexpected invalidatetiles request [" << tiles << "].");
+    return std::pair<int, Util::Rectangle>(-1, Util::Rectangle(0, 0, 0, 0));
 }
 
 void TileCache::removeFile(const std::string& fileName)
@@ -451,7 +498,7 @@ void TileCache::subscribeToTileRendering(const TileDesc& tile, const std::shared
 
     if (tileBeingRendered)
     {
-        for (const auto &s : tileBeingRendered->_subscribers)
+        for (const auto &s : tileBeingRendered->getSubscribers())
         {
             if (s.lock().get() == subscriber.get())
             {
@@ -462,8 +509,10 @@ void TileCache::subscribeToTileRendering(const TileDesc& tile, const std::shared
         }
 
         LOG_DBG("Subscribing " << subscriber->getName() << " to tile " << name << " which has " <<
-                tileBeingRendered->_subscribers.size() << " subscribers already.");
-        tileBeingRendered->_subscribers.push_back(subscriber);
+                tileBeingRendered->getSubscribers().size() << " subscribers already.");
+        tileBeingRendered->getSubscribers().push_back(subscriber);
+        if(tile.getId() == -1)
+            subscriber->traceSubscribeToTile(tileBeingRendered->getCacheName());
 
         const auto duration = (std::chrono::steady_clock::now() - tileBeingRendered->getStartTime());
         if (std::chrono::duration_cast<std::chrono::milliseconds>(duration).count() > COMMAND_TIMEOUT_MS)
@@ -482,7 +531,32 @@ void TileCache::subscribeToTileRendering(const TileDesc& tile, const std::shared
         assert(_tilesBeingRendered.find(cachedName) == _tilesBeingRendered.end());
 
         tileBeingRendered = std::make_shared<TileBeingRendered>(cachedName, tile);
-        tileBeingRendered->_subscribers.push_back(subscriber);
+        tileBeingRendered->getSubscribers().push_back(subscriber);
+        if(tile.getId() == -1)
+            subscriber->traceSubscribeToTile(tileBeingRendered->getCacheName());
+        _tilesBeingRendered[cachedName] = tileBeingRendered;
+    }
+}
+
+void TileCache::registerTileBeingRendered(const TileDesc& tile)
+{
+    std::shared_ptr<TileBeingRendered> tileBeingRendered = findTileBeingRendered(tile);
+    if (tileBeingRendered)
+    {
+        const auto duration = (std::chrono::steady_clock::now() - tileBeingRendered->getStartTime());
+        if (std::chrono::duration_cast<std::chrono::milliseconds>(duration).count() > COMMAND_TIMEOUT_MS)
+        {
+            // Tile painting has stalled. Reissue.
+            tileBeingRendered->setVersion(tile.getVersion());
+        }
+    }
+    else
+    {
+        const std::string cachedName = cacheFileName(tile);
+
+        assert(_tilesBeingRendered.find(cachedName) == _tilesBeingRendered.end());
+
+        tileBeingRendered = std::make_shared<TileBeingRendered>(cachedName, tile);
         _tilesBeingRendered[cachedName] = tileBeingRendered;
     }
 }
@@ -507,7 +581,7 @@ std::string TileCache::cancelTiles(const std::shared_ptr<ClientSession> &subscri
             continue;
         }
 
-        auto& subscribers = it->second->_subscribers;
+        auto& subscribers = it->second->getSubscribers();
         LOG_TRC("Tile " << it->first << " has " << subscribers.size() << " subscribers.");
 
         const auto itRem = std::find_if(subscribers.begin(), subscribers.end(),
@@ -535,7 +609,7 @@ std::string TileCache::cancelTiles(const std::shared_ptr<ClientSession> &subscri
 
 void TileCache::assertCorrectThread()
 {
-    const bool correctThread = _owner == std::thread::id(0) || std::this_thread::get_id() == _owner;
+    const bool correctThread = _owner == std::thread::id() || std::this_thread::get_id() == _owner;
     if (!correctThread)
         LOG_ERR("TileCache method invoked from foreign thread. Expected: " <<
                 Log::to_string(_owner) << " but called from " <<
