@@ -17,6 +17,7 @@
 #include <string>
 
 #include <Poco/Timestamp.h>
+#include <Rectangle.hpp>
 
 #include "TileDesc.hpp"
 
@@ -33,7 +34,7 @@ public:
     /// When the docURL is a non-file:// url, the timestamp has to be provided by the caller.
     /// For file:// url's, it's ignored.
     /// When it is missing for non-file:// url, it is assumed the document must be read, and no cached value used.
-    TileCache(const std::string& docURL, const Poco::Timestamp& modifiedTime, const std::string& cacheDir);
+    TileCache(const std::string& docURL, const Poco::Timestamp& modifiedTime, const std::string& cacheDir, bool tileCachePersistent);
     ~TileCache();
 
     /// Remove the entire cache directory.
@@ -45,6 +46,11 @@ public:
     /// Otherwise returns 0 to signify a subscription exists.
     void subscribeToTileRendering(const TileDesc& tile, const std::shared_ptr<ClientSession>& subscriber);
 
+    /// Create the TileBeingRendered object for the given tile indicating that the tile was sent to
+    /// the kit for rendering. Note: subscribeToTileRendering calls this internally, so you don't need
+    /// to call this method if you need also to subcribe for the rendered tile.
+    void registerTileBeingRendered(const TileDesc& tile);
+
     /// Cancels all tile requests by the given subscriber.
     std::string cancelTiles(const std::shared_ptr<ClientSession>& subscriber);
 
@@ -72,14 +78,23 @@ public:
     // The tiles parameter is an invalidatetiles: message as sent by the child process
     void invalidateTiles(const std::string& tiles);
 
+    /// Parse invalidateTiles message to a part number and a rectangle of the invalidated area
+    static std::pair<int, Util::Rectangle> parseInvalidateMsg(const std::string& tiles);
+
     /// Store the timestamp to modtime.txt.
     void saveLastModified(const Poco::Timestamp& timestamp);
 
-    void forgetTileBeingRendered(const TileDesc& tile);
+    void forgetTileBeingRendered(const std::shared_ptr<TileCache::TileBeingRendered>& tileBeingRendered, const TileDesc& tile);
+    double getTileBeingRenderedElapsedTimeMs(const std::string& tileCacheName) const;
+
+    bool hasTileBeingRendered(const TileDesc& tile);
+    int getTileBeingRenderedVersion(const TileDesc& tile);
 
     void setThreadOwner(const std::thread::id &id) { _owner = id; }
     void assertCorrectThread();
 
+
+
 private:
     void invalidateTiles(int part, int x, int y, int width, int height);
 
@@ -99,6 +114,8 @@ private:
 
     const std::string _cacheDir;
 
+    const bool _tileCachePersistent;
+
     std::thread::id _owner;
 
     std::map<std::string, std::shared_ptr<TileBeingRendered> > _tilesBeingRendered;
